package javax.swing.table;

import java.text.Collator;
import java.util.Comparator;
import javax.swing.DefaultRowSorter;

public class TableRowSorter<M extends TableModel> extends DefaultRowSorter<M, Integer>
{
    private static final Comparator COMPARABLE_COMPARATOR;
    private M tableModel;
    private TableStringConverter stringConverter;
    
    public TableRowSorter() {
        this(null);
    }
    
    public TableRowSorter(M model) {
        this.setModel(model);
    }
    
    public void setModel(M tableModel) {
        this.tableModel = tableModel;
        this.setModelWrapper(new TableRowSorter.TableRowSorterModelWrapper());
    }
    
    public void setStringConverter(TableStringConverter stringConverter) {
        this.stringConverter = stringConverter;
    }
    
    public TableStringConverter getStringConverter() {
        return this.stringConverter;
    }
    
    @Override
    public Comparator<?> getComparator(int column) {
        Comparator<Object> comparator = super.getComparator(column);
        if (comparator != null) {
            return comparator;
        }
        Class<Object> columnClass = ((TableModel)this.getModel()).getColumnClass(column);
        if (columnClass == String.class) {
            return Collator.getInstance();
        }
        if (Comparable.class.isAssignableFrom(columnClass)) {
            return TableRowSorter.COMPARABLE_COMPARATOR;
        }
        return Collator.getInstance();
    }
    
    @Override
    protected boolean useToString(int column) {
        if (super.getComparator(column) != null) {
            return false;
        }
        Class<Object> columnClass = ((TableModel)this.getModel()).getColumnClass(column);
        return columnClass != String.class && !Comparable.class.isAssignableFrom(columnClass);
    }
    
    static {
        COMPARABLE_COMPARATOR = new TableRowSorter.ComparableComparator();
    }
    
    private class TableRowSorterModelWrapper extends DefaultRowSorter.ModelWrapper<M, Integer>
    {
        @Override
        public M getModel() {
            return TableRowSorter.this.tableModel;
        }
        
        @Override
        public int getColumnCount() {
            return (TableRowSorter.this.tableModel == null) ? 0 : TableRowSorter.this.tableModel.getColumnCount();
        }
        
        @Override
        public int getRowCount() {
            return (TableRowSorter.this.tableModel == null) ? 0 : TableRowSorter.this.tableModel.getRowCount();
        }
        
        @Override
        public Object getValueAt(int n, int n2) {
            return TableRowSorter.this.tableModel.getValueAt(n, n2);
        }
        
        @Override
        public String getStringValueAt(int n, int n2) {
            TableStringConverter stringConverter = TableRowSorter.this.getStringConverter();
            if (stringConverter != null) {
                String string2 = stringConverter.toString(TableRowSorter.this.tableModel, n, n2);
                if (string2 != null) {
                    return string2;
                }
                return "";
            }
            else {
                Object value = this.getValueAt(n, n2);
                if (value == null) {
                    return "";
                }
                String string = value.toString();
                if (string == null) {
                    return "";
                }
                return string;
            }
        }
        
        @Override
        public Integer getIdentifier(int i) {
            return Integer.valueOf(i);
        }
    }
    
    private static class ComparableComparator implements Comparator
    {
        @Override
        public int compare(Object o, Object o2) {
            return ((Comparable)o).compareTo(o2);
        }
    }
}
